@@ -146,8 +146,8 @@ class ResourceHandler(BaseHandler):
             except (ParsingException, AttributeError) as pse:
                 raise HTTPError(400, str(pse))
 
-            key = self._create_key(entity)
             entity.owner = self.get_current_user()
+            key = self._create_key(entity)
             RESOURCES[key] = entity
             self._headers['Location'] = key
         self.write('OK')
@@ -240,9 +240,11 @@ class ResourceHandler(BaseHandler):
         @param entity: The entity to create the key for.
         @type entity: Entity
         '''
-        # FIXME: handle name-spaces here...
-        # pylint: disable=R0201
-        key = entity.kind.location + str(uuid.uuid4())
+        if entity.kind.location is not '':
+            key = '/users/' + self.get_current_user() + entity.kind.location
+            key += str(uuid.uuid4())
+        else:
+            key = '/users/' + self.get_current_user() + '/' + str(uuid.uuid4())
         entity.identifier = key
         return key
 
@@ -262,16 +264,70 @@ class ListHandler(BaseHandler):
         '''
         Returns a dict with all categories which have locations.
         '''
-        # FIXME: move this up to self.locations...
         locations = {}
         for cat in registry.BACKENDS.keys():
             if hasattr(cat, 'location') and cat.location is not '':
                 if hasattr(cat, 'owner') and cat.owner is not '':
-                    if cat.owner is not self.get_current_user():
+                    if cat.owner is self.get_current_user():
+                        locations[cat.location] = cat
                         break
                 locations[cat.location] = cat
         return locations
 
+    def get_my_resources(self):
+        '''
+        Returns a list of all resources belonging to the current user.
+        '''
+        my_resources = []
+        for tmp in RESOURCES.keys():
+            item = RESOURCES[tmp]
+            if item.owner == self.get_current_user():
+                my_resources.append(item)
+        return my_resources
+
+    def get_all_resources_of_category(self, category, resources):
+        '''
+        Return all resources belonging to one category.
+        
+        @param category: A category
+        @type category: Category
+        @param resources: List of resources
+        @type resources: list
+        '''
+        tmp = []
+        for res in resources:
+            if res.kind == category:
+                tmp.append(res)
+            elif category in res.mixins:
+                tmp.append(res)
+        return tmp
+
+    def filter_childs(self, key, resources, categories):
+        '''
+        Retrieve childs in a hierachy and use categories to filter the result.
+        
+        @param key: A key.
+        @type key: str
+        @param resources: A list of resources.
+        @type resources: list
+        @param categories: A list of categories which is used to filter.
+        @type categories: list
+        '''
+        tmp = []
+        for name in resources:
+            if name.identifier.find(key) > -1 and key.endswith('/'):
+                if categories is None:
+                    tmp.append(name)
+                elif len(categories) > 0:
+                    for category in categories:
+                        if category == name.kind:
+                            tmp.append(name)
+                        elif category in name.kind.related:
+                            tmp.append(name)
+                        elif category in name.mixins:
+                            tmp.append(name)
+        return tmp
+
     @tornado.web.authenticated
     def get(self, key):
         key = '/' + key + '/'
@@ -282,36 +338,22 @@ class ListHandler(BaseHandler):
         try:
             data_parser = self.get_pyocci_parser('Content-Type')
             categories = data_parser.to_categories(headers, body)
+            print categories
         except (KeyError, ParsingException):
             pass
 
         locations = self.get_locations()
         resources = []
 
-        my_resources = {}
-        for tmp in RESOURCES.keys():
-            item = RESOURCES[tmp]
-            if item.owner == self.get_current_user():
-                my_resources[tmp] = item
+        my_resources = self.get_my_resources()
+
+        if key in locations:
+            tmp = self.get_all_resources_of_category(locations[key],
+                                                     my_resources)
+            resources = self.filter_childs('/', tmp, categories)
+        elif key.endswith('/'):
+            resources = self.filter_childs(key, my_resources, categories)
 
-        for name in my_resources.keys():
-            res = my_resources[name]
-            if key in locations:
-                if res.kind == locations[key]:
-                    resources.append(res)
-                elif locations[key] in res.mixins:
-                    resources.append(res)
-            elif name.find(key) > -1 and key.endswith('/'):
-                if categories is None:
-                    resources.append(res)
-                elif len(categories) > 0:
-                    for category in categories:
-                        if category == res.kind:
-                            resources.append(res)
-                        elif category in res.kind.related:
-                            resources.append(res)
-                        elif category in res.mixins:
-                            resources.append(res)
         if len(resources) > 0:
             heads, data = return_parser.from_entities(resources)
             return self._send_response(heads, data)
@@ -326,6 +368,7 @@ class ListHandler(BaseHandler):
         parser = self.get_pyocci_parser('Content-Type')
 
         locations = self.get_locations()
+        print locations
 
         if key in locations:
             try:
@@ -368,6 +411,7 @@ class QueryHandler(BaseHandler):
     '''
 
     # TODO: check for adding one location twice...
+    # TODO: filter categories...
 
     # disabling 'Too many public methods' pylint check (tornado's fault)
     # pylint: disable=R0904
